import os
import json

import gradio as gr

import llm_zhipuai
import utils


doc_path = './data/all_doc_datas.json'
history_path = './data/history/'
daybook_path = './data/daybook/'

with open('maomao.txt', 'r', encoding='utf-8') as f:
    system_prompt = f.read()

# 读取档案
all_doc_datas = {}
if os.path.exists(doc_path):
    with open(doc_path, 'r', encoding='utf-8') as f:
        all_doc_datas = json.load(f)
    f.close()
print('【all_doc_datas】', all_doc_datas)


# 历史对话加载
all_his_datas = {}
file_list = os.listdir(history_path)
for item in file_list:
    username = item[:-5]
    with open(history_path + item, 'r', encoding='utf-8') as f:
        all_his_datas[username] = f.read().split('\n')[:-1]
        all_his_datas[username] = [eval(it) for it in all_his_datas[username]]
    f.close()

print('【all_his_datas】', all_his_datas)


# 历史日记加载
all_day_datas = {}
file_list = os.listdir(daybook_path)
for item in file_list:
    username = item[:-5]
    with open(daybook_path + item, 'r', encoding='utf-8') as f:
        all_day_datas[username] = f.read().split('\n')[:-1]
        all_day_datas[username] = [eval(it) for it in all_day_datas[username]]
    f.close()

print('【all_day_datas】', all_day_datas)


# 获取档案历史对话
def get_his(doc_name, his_datas):
    chatbot = []
    history = []

    if doc_name in his_datas.keys():
        history = his_datas[doc_name]
        temp_his = ['', '']
        for item in his_datas[doc_name]:
            if item['role'] == 'user':
                if len(temp_his[0]) == 0:
                    temp_his[0] = item['content']
                else:
                    chatbot.append((temp_his[0], temp_his[1]))
                    temp_his = [item['content'], '']
            elif item['role'] == 'assistant':
                if len(temp_his[1]) == 0:
                    temp_his[1] = item['content']
                else:
                    chatbot.append((temp_his[0], temp_his[1]))
                    temp_his = ['', item['content']]

            if len(temp_his[0]) > 0 and len(temp_his[1]) > 0:
                chatbot.append((temp_his[0], temp_his[1]))
                temp_his = ['', '']
        if len(temp_his[0]) > 0 or len(temp_his[1]) > 0:
            chatbot.append((temp_his[0], temp_his[1]))
    return chatbot, history

# 读取档案
def doc_load(doc_name):
    chatbot, chat_his = get_his(doc_name, all_his_datas)
    daybook, day_his = get_his(doc_name, all_day_datas)

    output = []

    if doc_name in all_doc_datas.keys():
        output.extend(all_doc_datas[doc_name] + [all_doc_datas[doc_name]])
    else:
        output.extend(['', '', '', '', '', '', '', '', '', []])

    output.extend([chatbot, chat_his, daybook, day_his])


    return output

# 保存档案
def doc_save(doc0, doc1, doc2, doc3, doc4, doc5, doc6, doc7, doc8):
    save_data = [doc0, doc1, doc2, doc3, doc4, doc5, doc6, doc7, doc8]
    print('【save_data】', save_data)
    all_doc_datas[doc0] = save_data
    with open(doc_path, 'w', encoding='utf-8') as f2:
        json.dump(all_doc_datas, f2, ensure_ascii=False, indent=2)
    f2.close()

    chatbot, chat_his = get_his(doc0, all_his_datas)
    daybook, day_his = get_his(doc0, all_day_datas)


    return save_data, chatbot, chat_his, daybook, day_his







# 保存聊天对话
def save_his_data(username, save_path, user_input=None, response=None):

    if username not in all_his_datas.keys():
        all_his_datas[username] = []

    # 保存数据
    with open(save_path + username + '.json', 'a', encoding='utf-8') as f2:
        if user_input is not None and len(user_input) > 0:
            all_his_datas[username].append(user_input)
            json.dump(user_input, f2, ensure_ascii=False)
            f2.write('\n')
        # 保存数据
        if response is not None and len(response) > 0:
            all_his_datas[username].append(response)
            json.dump(response, f2, ensure_ascii=False)
            f2.write('\n')
    f2.close()


def predict(username, user_input, doc_list, history, food, feel, energy, chatbot):
    global system_prompt, system
    # print(history)
    system = system_prompt
    for i in range(len(doc_list)-1):
        system = system.replace("【doc{}】".format(i+1), doc_list[i+1])

    state = '\n\n当前宠物状态（取值0-100）：\n饱腹  {}\n心情  {}\n精力  {}'.format(food, feel, energy)

    system += state + '\n当前时间：{}'.format(utils.get_now_datetime())
    print('【system】', system)
    # print('【history】', history)
    chatbot.append((user_input, ""))
    input_time = utils.get_now_datetime()
    if len(doc_list) > 0:
        out_response = ''
        for response in llm_zhipuai.get_stream_chat(prompt=user_input,
                                                    system=system,
                                                    history=history[:-10]):
            # print(response)
            chatbot[-1] = (user_input, response)

            new_history = history + [{'role': 'user', 'content': user_input, 'time': input_time},
                                      {'role': 'assistant', 'content': response, 'time': utils.get_now_datetime()}]
            out_response = response
            yield chatbot, new_history

        save_his_data(username, history_path, {'role': 'user', 'content': user_input, 'time': input_time}, {'role': 'assistant', 'content': out_response, 'time': utils.get_now_datetime()})
        print('保存对话')
    else:
        chatbot[-1] = (user_input, '请完善宠物档案，当前未填写')
        yield chatbot, history


def reset_user_input():
    return gr.update(value='')


def reset_state():
    return []





# ===========日记==============
def daybook_save(day_input, daybook_history, daybook):
    input_time = utils.get_now_datetime()
    daybook.append((day_input, input_time))
    daybook_history += [{'role': 'user', 'content': day_input, 'time': input_time},
                        {'role': 'assistant', 'content': input_time, 'time': input_time}]

    save_his_data(username,
                  daybook_path,
                  {'role': 'user', 'content': day_input, 'time': input_time},
                  {'role': 'assistant', 'content': input_time, 'time': input_time})

    return daybook, daybook_history



with gr.Blocks() as demo:
    with gr.Tab('档案'):
        gr.HTML("""<h1 align="center">宠物档案</h1>""")
        gr.HTML("""<h4 align="center">请先为您家宠物建立自己的档案，再使用上方其他功能~</h1>""")

        with gr.Row():
            doc_name = gr.Textbox(show_label=False, placeholder="若已建过档，请输入昵称", lines=1, scale=2)
            doc_search_but = gr.Button("读档", variant="primary", scale=1)

        gr.HTML("""<h3 align="left">0. 用户名，作为后续查询唯一ID，请自己记住</h3>""")
        doc0 = gr.Textbox(show_label=False, placeholder="示例：诗诗", lines=1)
        gr.HTML("""<h3 align="left">1. 宠物的昵称，平时你会怎么叫你家宠物？</h3>""")
        doc1 = gr.Textbox(show_label=False, placeholder="示例：诗诗、小狮子（因为她脑袋上的毛长长的，像一只小狮子）", lines=1)
        gr.HTML("""<h3 align="left">2. 宠物的性别是什么？</h3>""")
        doc2 = gr.Textbox(show_label=False, placeholder="示例：母", lines=1)
        gr.HTML("""<h3 align="left">3. 宠物属于什么品种？</h3>""")
        doc3 = gr.Textbox(show_label=False, placeholder="示例：是一只小土狗，我也不知道什么品种", lines=1)
        gr.HTML("""<h3 align="left">4. 宠物的颜色是什么，花纹有什么特别之处？</h3>""")
        doc4 = gr.Textbox(show_label=False, placeholder="示例：棕褐色，脑袋上的毛像雄狮的鬃毛，肚皮上的毛是浅色的，摸起来很软", lines=1)
        gr.HTML("""<h3 align="left">5. 宠物的性格怎么样，有什么小个性？</h3>""")
        doc5 = gr.Textbox(show_label=False, placeholder="示例：非常亲近我，喜欢跳起来和我玩。听力非常好，我很远叫它都可以马上听到，跑过来很快很快。什么都爱吃，没心没肺的。出去玩就会围着我绕圈跑。对我永远热情。对外人有一小点怕生。很聪明~教东西学的很快，会自己上厕所，听得懂指令。", lines=1)
        gr.HTML("""<h3 align="left">6. 宠物的爱好是什么？平时都是怎么样过一天？</h3>""")
        doc6 = gr.Textbox(show_label=False, placeholder="示例：没有什么特别的爱好，喜欢跑来跑去，在院子里刨东西。我去上学了，它就会在家里等我。每次远远地叫一声，就会冲过来围着我转一起回家。有时候我妈妈会开电动载它去学校找我，一起等我放学。一路就兜兜转转回去。", lines=1)
        gr.HTML("""<h3 align="left">7. 宠物最喜欢吃的是什么、有什么喜欢的玩具等？</h3>""")
        doc7 = gr.Textbox(show_label=False, placeholder="示例：我会喂它吃鸡腿，它很爱吃。剩菜剩饭都吃，感觉没有什么挑食的。没有给它吃过狗粮，不知道会不会喜欢吃。因为运动量大，经常要喝水。我还没给它买过玩具，最喜欢到处跑。特别是去大草坪撒开跑~", lines=1)
        gr.HTML("""<h3 align="left">8. 你希望宠物怎么称呼你？</h3>""")
        doc8 = gr.Textbox(show_label=False, placeholder="示例：主人", lines=1)

        with gr.Column(min_width=32, scale=1):
            doc_save_btn = gr.Button("建档", variant="primary")

    with gr.Tab('聊天'):
        gr.HTML("""<h1 align="center">宠物陪伴</h1>""")
        chatbot = gr.Chatbot()
        with gr.Row():
            with gr.Column(scale=4):
                with gr.Column(scale=12):
                    user_input = gr.Textbox(show_label=False, placeholder="请输入...", lines=2)
                with gr.Column(min_width=32, scale=1):
                    submitBtn = gr.Button("发送", variant="primary")
            with gr.Column(scale=1):
                emptyBtn = gr.Button("忘记历史内容")
                food = gr.Slider(0, 100, value=70, step=1, label="饱腹", interactive=True)
                feel = gr.Slider(0, 100, value=85, step=1, label="心情", interactive=True)
                energy = gr.Slider(0, 100, value=85, step=1, label="精力", interactive=True)
                # max_length = gr.Slider(0, 4096, value=2048, step=1.0, label="Maximum length", interactive=True)
                # top_p = gr.Slider(0, 1, value=0.01, step=0.01, label="Top P", interactive=True)
                # temperature = gr.Slider(0, 1, value=0.01, step=0.01, label="Temperature", interactive=True)

    with gr.Tab('日记'):
        gr.HTML("""<h1 align="center">宠物日记</h1>""")
        daybook = gr.Chatbot()
        with gr.Row():
            with gr.Column(scale=12):
                daybook_input = gr.Textbox(show_label=False, placeholder="请输入...", lines=2)
            with gr.Column(min_width=32, scale=1):
                daybook_submitBtn = gr.Button("发送", variant="primary")


    history = gr.State([])
    doc_list = gr.State([])
    daybook_history = gr.State([])

    # 档案读取
    doc_search_but.click(doc_load,
                         [doc_name],
                         [doc0, doc1, doc2, doc3, doc4, doc5, doc6, doc7, doc8, doc_list, chatbot, history, daybook, daybook_history])
    # 档案保存
    doc_save_btn.click(doc_save,
                       [doc0, doc1, doc2, doc3, doc4, doc5, doc6, doc7, doc8],
                       [doc_list, chatbot, history, daybook, daybook_history])


    # 聊天对话
    submitBtn.click(predict,
                    [doc0, user_input, doc_list, history, food, feel, energy, chatbot],
                    [chatbot, history])
    submitBtn.click(reset_user_input, [], [user_input])

    emptyBtn.click(reset_state, outputs=[history])


    # 日记
    daybook_submitBtn.click(daybook_save,
                            [daybook_input, daybook_history, daybook],
                            [daybook, daybook_history])
    daybook_submitBtn.click(reset_user_input, [], [daybook_input])

# , server_name='192.168.62.184'
demo.queue().launch(share=False, inbrowser=True, server_name='192.168.62.184', server_port=5001)